// Copyright (C) 2022 - 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#include <cstring>
#include "xrm_dec_interface.h"
#include "xrm_utils.h"
#include "log_ama.h"

/**
 * @brief Release the xrm resources which were allocated. Destroy the xrm 
 * context.
 * 
 * @param xrm_dec_ctx: The xrm decoder context
 */
void xrm_dec_release(XrmDecodeContext* xrm_dec_ctx) {
  if (!xrm_dec_ctx) {
    return;
  }
  if (xrm_dec_ctx->decode_res_in_use) {
    /* Release the resource (still reserved) */
    xrmCuListReleaseV2(xrm_dec_ctx->xrm_ctx, &xrm_dec_ctx->decode_cu_list_res);
    xrm_dec_ctx->decode_res_in_use = 0;
  }
  xrmDestroyContext(xrm_dec_ctx->xrm_ctx);
}

/**
 * @brief Check & allocate the necessary xrm resources given the relevant 
 * decoder parameters
 * 
 * @param xrm_dec_ctx Used to store information about resources this function
 * allocates so they can be safely released later
 * @param dev_index The device which will be queried
 * @param xrm_props Contains relevant information for load calculation
 * @return XRM_SUCCESS on success XRM_ERROR on error
 */
int xrm_dec_reserve(XrmDecodeContext* xrm_dec_ctx, int dev_index, XrmInterfaceProperties* xrm_props) {
  xrmCuPoolPropertyV2 dec_cu_pool_prop;
  int                 ret = XRM_ERROR;
  memset(&dec_cu_pool_prop, 0, sizeof(dec_cu_pool_prop));

  xrm_dec_ctx->dev_index = dev_index;

  /* Create XRM context */
  xrm_dec_ctx->xrm_ctx = xrmCreateContext(XRM_API_VERSION_1);
  if (xrm_dec_ctx->xrm_ctx == NULL) {
    return XRM_ERROR_CONNECT_FAIL;
  }

  /* Calculate decoder load based on decoder properties */
  ret = xrm_utils_load_calc(xrm_dec_ctx->xrm_ctx, xrm_props, NULL, 0, "DECODER", &xrm_dec_ctx->dec_load, 0);
  if (ret <= XRM_ERROR) {
    LOG_AMA_ERROR(LOG_AMA_LAYER_XRM, "Decoder load calculation failed %d", ret);
    return XRM_ERROR;
  }
  xrm_utils_update_pool_props("DECODER", xrm_dec_ctx->dev_index, xrm_dec_ctx->dec_load, &dec_cu_pool_prop, 0);
  ret                            = xrm_utils_reserve_resources(xrm_dec_ctx->xrm_ctx, &dec_cu_pool_prop, &xrm_dec_ctx->decode_cu_list_res);
  xrm_dec_ctx->decode_res_in_use = ret == XRM_SUCCESS ? 1 : 0;
  xrm_dec_ctx->dev_index         = xrm_dec_ctx->decode_cu_list_res.cuResources[0].deviceId;
  return ret;
}
